﻿/*
 * COMMON.c
 *
 * Created: 2013/10/10 21:37:49
 *  Author: Koichi Nishida
 */ 
/*
Copyright (C) 2013 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include "BUFFER.h"
#include "COMMON.h"
#include "SD.h"
#include "LCD.h"
#include "DISK2.h"
#include "SMART.h"

// mode : DISK2MODE | SMARTMODE
enum MODE mode;

// whether FlashAir or not
uint8_t isFlashAir = 0;

// decremented toward 0 by a 0.01 sec counter.
volatile uint16_t timerOneShot, timerOneShot2;
// blink at 0.33 sec
volatile uint8_t timerBlink = 0;

// shared variable between DISK2 mode and SMART mode
union SVAR svar;

// initialize
void COMMON_init(void)
{
	// initialize input ports
	// PHASE
	PORTA.DIRCLR = PHASE_bm;
	PORTCFG.MPCMASK = PHASE_bm;
	PORTA.PIN0CTRL = PORT_OPC_PULLUP_gc;
	// WDAT	
	PORTA.DIRCLR = PIN6_bm;
	PORTA.PIN6CTRL = PORT_OPC_PULLUP_gc;
	// WREQ
	PORTD.DIRCLR = PIN2_bm;
	PORTD.PIN2CTRL = PORT_OPC_PULLUP_gc;
	// EN2
	PORTD.DIRCLR = PIN5_bm;
	PORTD.PIN5CTRL = PORT_OPC_PULLUP_gc;
	// EN1
	PORTD.DIRCLR = PIN6_bm;
	PORTD.PIN6CTRL = PORT_OPC_PULLUP_gc;
	// HDSEL
	PORTA.DIRCLR = PIN7_bm;
	PORTA.PIN7CTRL = PORT_OPC_PULLUP_gc;
	// DIK35
	PORTD.DIRCLR = PIN4_bm;
	PORTD.PIN4CTRL = PORT_OPC_PULLUP_gc;

	// initialize output ports
	// RDR
	PORTD.DIRSET = PIN0_bm;
	// SENSE
	PORTD.DIRSET = PIN7_bm;

	// PHASE interrupt
	PORTA.PIN0CTRL |= PORT_ISC_BOTHEDGES_gc;
	PORTA.INTMASK |= PHASE_bm;
	PORTA.INTCTRL = PORT_INTLVL_LO_gc;
	PMIC.CTRL = PMIC_LOLVLEN_bm; 

	// TCC5 about 0.01 sec used by UI
	TCC5.PER = 313;
	TCC5.CTRLA = TC5_CLKSEL2_bm|TC5_CLKSEL1_bm|TC5_CLKSEL0_bm;			// prescaler : clk/1024
	TCC5.INTCTRLA = 0;	// meddle level interrupt, but off for the time being
	PMIC.CTRL = PMIC_LOLVLEN_bm;

	// TCD5 about 1 sec used for FlashAir
	TCD5.PER = 31250;
	TCD5.CTRLA = TC5_CLKSEL2_bm|TC5_CLKSEL1_bm|TC5_CLKSEL0_bm;			// prescaler : clk/1024
	TCD5.INTCTRLA = 0;

	if (mode == DISK2MODE)
		for (uint8_t i = 0; i < 2; i++) buffer2.disk2.img[i].valid = 0;
	else
		for (uint8_t i = 0; i < 4; i++) buffer2.smart.img[i].valid = 0;
}

uint32_t readmem_long(uint8_t *mem)
{
	return *(uint32_t *)mem;
}

uint16_t readmem_word(uint8_t *mem)
{
	return *(uint16_t *)mem;
}

// timer handler
ISR(TCC5_OVF_vect) {
	TCC5_INTFLAGS = 1;
	static uint8_t counter = 0;
	if (timerOneShot>0) timerOneShot--;
	if (timerOneShot2>0) timerOneShot2--;
		
	if (++counter==33) {counter = 0; timerBlink ^= 1;}
}

// convert a '0'-'F' to 0x0 - 0xf
uint8_t to16(uint8_t a)
{
	if (a>='0' && a<='9') return (a-'0');
	if (a>='A' && a<='F') return (a-'A'+10);
	if (a>='a' && a<='f') return (a-'a'+10);
	return 0;	
}

