﻿/*
 * COMMON.h
 *
 * Created: 2013/09/23 14:11:33
 *  Author: Koichi Nishida
 */ 
/*
Copyright (C) 2013 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef COMMON_H_
#define COMMON_H_

#include "SD.h"

#define NONE 0

#define DISK2_WRITE_BUF_NUM 5
#define SMART_PARTITION_MAX 4

// mode : DISK2MODE | SMARTMODE
enum MODE {DISK2MODE, SMARTMODE};
extern enum MODE mode;

// whether FlashAir or not
extern uint8_t isFlashAir;

// decremented by a 0.01 sec counter. used by push buttons.
extern volatile uint16_t timerOneShot, timerOneShot2;
extern volatile uint8_t timerBlink;

// ON/OFF 0.01 sec timer
#define ON_TIMER2 TCC5.INTCTRLA = TC5_OVFINTLVL0_bm;
#define OFF_TIMER2 TCC5.INTCTRLA = 0;

#define ON_PHASEINT PORTA.INTCTRL = PORT_INTLVL_LO_gc;
#define OFF_PHASEINT { PORTA.INTCTRL = 0; }

#define PHASE_bm 0b00001111

#define DISABLE_CS PORTC_OUTSET=SPI_SS_bm
#define ENABLE_CS PORTC_OUTCLR=SPI_SS_bm

#define PHASE (/*PORTA_IN*/VPORT0_IN&0b00001111)
#define WDAT (/*PORTA_IN*/VPORT0_IN&PIN6_bm)
#define WREQ (/*PORTD_IN*/VPORT2_IN&PIN2_bm)
#define EN2 (/*PORTD_IN*/VPORT2_IN&PIN5_bm)
#define EN1 (/*PORTD_IN*/VPORT2_IN&PIN6_bm)
#define HDSEL (/*PORTA_IN*/VPORT0_IN&PIN7_bm)
#define DIK35 (/*PORTD_IN*/VPORT2_IN&PIN4_bm)

#define EEP_MODE (uint8_t *)0x0001

#define nop() __asm__ __volatile__ ("nop");

// image files
extern struct FILE img[];

// shared variable between DISK2 mode and SMART mode
union SVAR {
	struct {
		// DISK II properties
		volatile uint8_t ph_track[2];				// physical track numbers : 0 - 139
		uint8_t volume[2];							// disk volumes
		volatile uint8_t currentDrive;				// 0 : drive1, 1 : drive2
		volatile uint8_t sector;					// sector : 0 - 15
		uint8_t formatting;							// 1 if formatting
		uint8_t sectors[DISK2_WRITE_BUF_NUM];
		uint8_t drives_tracks[DISK2_WRITE_BUF_NUM];	// for remembering written sectors, drives & tracks
		volatile uint8_t WrtBuffNum;				// write buffer number
		uint8_t imageReady;
	} DISK2;
	struct {
		struct FILE *image[SMART_PARTITION_MAX];	//disk images
		uint8_t device_id[SMART_PARTITION_MAX];		//to hold assigned device id's for the partitions
		uint8_t partition_num;
		uint8_t number_partitions_initialised;
		uint8_t desktop;
		uint8_t is2mg[4];
	} SMART;
};

extern union SVAR svar;
// common initialization
void COMMON_init(void);

// read a long word data from memory
uint32_t readmem_long(uint8_t *mem);

// read a word data from memory
uint16_t readmem_word(uint8_t *mem);

// convert a '0'-'F' to 0x0 - 0xf
uint8_t to16(uint8_t a);

#endif /* COMMON_H_ */