﻿/*
 * FLASHAIR.c
 *
 * Created: 2014/10/21 22:42:32
 *  Author: 浩一
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "SD.h"
#include "BUFFER.h"
#include "UI.h"
#include "LCD.h"
#include "COMMON.h"
#include "DISK2.h"

// examine whether the card is FlashAir or not. must be called right after the first SD initialization
uint8_t IS_FLASHAIR(void)
{
	uint8_t err = 0;

	char *config = (char *)buffer2.wifi.short_name;
	strcpy (config, "SD_WLAN    CONFIG     ");	
	// open the file with the absolute path
	// name should be <8 character directory name><3 character extension><8...><3...>...0
	FILE_openAbs(&buffer2.wifi.img[0], config, 1, &err);
	return !err;
}

void FLASHAIR_set_status(void)
{
	uint8_t err = 0;

	switch (mode) {
		case DISK2MODE:	{
			OFF_PHASEINT;
			SD_writeSharedMemoryBegin(0, 512, &err);
			// 0x0054 = 'T' = status command
			SPI_writeByte('T', &err);
			SPI_writeByte(mode, &err);
			SPI_writeByte(svar.DISK2.ph_track[0]>>2, &err);
			SPI_writeByte(svar.DISK2.ph_track[1]>>2, &err);
			SPI_writeByte(!EN1, &err);
			SPI_writeByte(!EN2, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			for (uint8_t drv=0; drv<2; drv++) {
				struct FILE *imgp = &buffer2.disk2.img[drv];
				if (imgp->valid) {
					for (uint8_t i=0; i<8; i++) SPI_writeByte(imgp->name[i], &err);
				} else {
					for (uint8_t i=0; i<8; i++) SPI_writeByte(' ', &err);
				}
			}
			for (uint16_t i=0; i<(512-24); i++) SPI_writeByte(0, &err);
			SD_writeSharedMemoryEnd(&err);
			ON_PHASEINT;
			break;
		}
		case SMARTMODE: {
			OFF_PHASEINT;
			SD_writeSharedMemoryBegin(0, 512, &err);
			// 0x0054 = 'T' = status command
			SPI_writeByte('T', &err);
			SPI_writeByte(mode, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			for (uint8_t drv=0; drv<4; drv++) {
				struct FILE *imgp = &buffer2.smart.img[drv];
				if (imgp->valid) {
					for (uint8_t i=0; i<8; i++) SPI_writeByte(imgp->name[i], &err);
				} else {
					for (uint8_t i=0; i<8; i++) SPI_writeByte(' ', &err);
				}
			}
			for (uint16_t i=0; i<(512-40); i++) SPI_writeByte(0, &err);
			SD_writeSharedMemoryEnd(&err);
			ON_PHASEINT;
			break;
		}
	}
}

// check and execute a command from shared memory
uint8_t FLASHAIR_command(void)
{
	uint8_t err = 0;
	uint8_t ret = 0;

	// read first word (command)
	OFF_PHASEINT;
	SD_readSharedMemoryBegin(0, 512, &err);
	uint8_t cmd = SPI_readByte(&err);
	uint8_t drive = SPI_readByte(&err)-'0';
	for (uint16_t i=0; i<510; i++) SPI_readByte(&err);
	SD_readSharedMemoryEnd(&err);
	ON_PHASEINT;
	
	switch (cmd) {
		// drive change command
		case 'D': {
			OFF_PHASEINT;
			FILE_getShortFileName(buffer2.wifi.short_name, &err);
			ON_PHASEINT;
			if (*(char *)buffer2.wifi.short_name) ret = UI_checkExecute(1, drive);
			break;
		}
		// unmount command
		case 'U': {
			strcpy((char *)buffer2.wifi.short_name, "           ");
			ret = UI_checkExecute(1, drive);
			break;	
		}
		// write back command
		case 'W': {
			// write back to .dsk file
			UI_checkExecute(2, drive);
			break;
		}
		// reset command
		case 'R': {
			FLASHAIR_set_status();
			cli();
			LCD_locate(0,0);
			eeprom_busy_wait();
			switch (drive) {
				case 0:
					eeprom_write_byte(EEP_MODE, DISK2MODE);
					eeprom_busy_wait();
					LCD_printAll("RESET asDISK2Drv");
					break;
				case 1:
					eeprom_write_byte(EEP_MODE, SMARTMODE);
					eeprom_busy_wait();
					LCD_printAll("RESET asSmartDrv");
					break;
			}
			_delay_ms(100);
			// RESET !
			CPU_CCP=CCP_IOREG_gc;
			RST.CTRL=RST_SWRST_bm;
			while (1);
		}
	}
	FLASHAIR_set_status();
	return ret;
}